/*
  Copyright (C) MOXA Inc. All rights reserved.
  This software is distributed under the terms of the
  MOXA License.  See the file COPYING-MOXA for details.
*/

/*******************************************************************************
File name:	udp2.c
 
Description:
	This program demo 
	how to use socket lib coding UDP send/recv program on UC7000 series.
	
	The Hardware connection is following:
	[UC7000 series]--- ~~LAN~~ ---PC Host Example (Windows platform)
	
	If data received from Ethernet,send these all back to the LAN port.
	If received 'q', the program will disconnect and close socket.
	1. Create UDP socket and receive data on Port 4001. 
	2. R/W data. write data to destination IP.
	3. if receive 'q', disconnect.
	
Usage:
    	1. Make downloading file(udp2-release or udp2-debug) for UC7000 series 
    	    1.1 Create executable file(udp2-release or udp2-debug) from source code(udp2.c) 
    	    	1.1.1 Modify udp2.c: 
    	    		change "DEST_IP" following the PC Host's IP address.
    	    		change "UDPPORT" if needed.	
    	    	1.1.2 Compiler and link to make udp2-release (no debug message) or udp2-debug (with debug message) 
	2. Download udp2-release or udp2-debug to UC7000 series 
     	    2.1 Using ftp download udp2-release or udp2-debug to a UC7000 series unit.    
    	3. Run Application on UC7000 series 
    	4. Execute UDP example for PC.[udpclient.exe or frmUdp.exe]
    	   input the UDP Port number equal to "UDPPORT"
    	         Destination ip equal to Network Enabler PCG's IP adddress.   
    	5. user can check if the sended data received back. 
    	6. Type 'q' to disconnect.    	    	          
    	    
History:    
    	Version		Author		Date		Comment
        1.0		Victor Yu.	01-16-2004      Wrote it.
*******************************************************************************/
#include        <stdio.h>
#include        <stdlib.h>
#include        <sys/types.h>
#include        <sys/socket.h>
#include        <sys/fcntl.h>
#include        <termios.h>
#include        <netinet/in.h>
#include        <errno.h>

#define DEST_IP "192.168.4.140"
#define UDPPORT	4001

#define TESTLEN	1024

char buf[TESTLEN];

void sdkprint(char *buf, unsigned short val);

int main()
{
	int			sfd, size, flag;
	int 			i, ret;
	struct sockaddr_in 	from, to;

	printf("Create socket......");
	if ((sfd=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0 ){
		printf("Fail %d\n", errno);
		return 0;
	} else 
		printf("OK\n");

	from.sin_addr.s_addr = htonl(INADDR_ANY);
	from.sin_family = AF_INET;
	from.sin_port = htons(UDPPORT);
 
	to.sin_addr.s_addr = inet_addr(DEST_IP);
	to.sin_family = AF_INET;
	to.sin_port = htons(UDPPORT);

	printf("Bind......");
	if (bind(sfd, (struct sockaddr *)&from, sizeof(from)) == 0)
		printf("OK\n");
	else {
		close(sfd);
		printf("Fail %d\n", errno);
		return 0;
        }

        printf("Ready to recv & write data\n");
	size = sizeof(to);
	flag = 1;
	while(flag) {
		if ((ret=recvfrom(sfd, buf, TESTLEN, 0,(struct sockaddr *)&from,&size)) > 0 ) {
			if (sendto(sfd, buf, ret, 0, (struct sockaddr *)&to, size) <= 0 ) {
				printf("sendto fail [%d]\n", errno);
				break;
			}
			for (i=0; i<ret; i++) {
				if (buf[i] == 'q') {
					printf("Receive 'q' and close socket\n");	
					flag = 0;	
				}					
			}
		} else if(ret < 0 ) {
			printf("recvfrom fail [%d]\n", errno);
			break;
		} 
	}
	
	printf("Close socket and exit program\n");	
	close(sfd);
	return 0;
}
